@@ -24,6 +24,9 @@
 #include <stdbool.h>
 
 extern bool interactive_mode;
+extern bool map_i_nl_cr;
+extern bool map_i_cr_nl;
+extern bool map_ign_cr;
 
 void stdout_configure(void);
 void stdin_configure(void);
